==> application-blocks-security-1.3.1-sources.jar: org/genesys/blocks/security/serialization/CurrentPermissionsWriter.java <==
/*
 * Copyright 2017 Global Crop Diversity Trust
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.genesys.blocks.security.serialization;

import com.fasterxml.jackson.core.JsonGenerator;
import com.fasterxml.jackson.databind.JavaType;
import com.fasterxml.jackson.databind.SerializerProvider;
import com.fasterxml.jackson.databind.cfg.MapperConfig;
import com.fasterxml.jackson.databind.introspect.AnnotatedClass;
import com.fasterxml.jackson.databind.introspect.BeanPropertyDefinition;
import com.fasterxml.jackson.databind.ser.VirtualBeanPropertyWriter;
import com.fasterxml.jackson.databind.util.Annotations;

import org.genesys.blocks.security.model.AclAwareModel;
import org.genesys.blocks.util.CurrentApplicationContext;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.BeansException;
import org.springframework.context.ApplicationContext;
import org.springframework.security.access.PermissionEvaluator;
import org.springframework.security.acls.domain.BasePermission;
import org.springframework.security.core.Authentication;
import org.springframework.security.core.context.SecurityContextHolder;

/**
 * The CurrentPermissionsWriter is applied to {@link AclAwareModel} and it
 * instructs Jackson to to include {@link CurrentPermissions} for current SID
 * for every ACL aware entity.
 * 
 * Serialization is enabled <code>@JsonAppend</code> annotation on
 * <code>AclAwareModel</code>:
 * 
 * <pre>
 *  &#64;JsonAppend(props = { @JsonAppend.Prop(name="_permissions", value = CurrentPermissionsWriter.class, type=CurrentPermissions.class) })
 *  public interface AclAwareModel...
 * </pre>
 * 
 * To be able to access the current permissions of the current SID, this code
 * relies on an instance of
 * <code>org.genesys.blocks.util.CurrentApplicationContext</code> to be
 * registered in the Spring application context.
 * 
 */
public class CurrentPermissionsWriter extends VirtualBeanPropertyWriter {

	/** The Constant LOG. */
	private static final Logger LOG = LoggerFactory.getLogger(CurrentPermissionsWriter.class);

	/** The Constant serialVersionUID. */
	private static final long serialVersionUID = 1L;

	/** The permission evaluator. */
	private static PermissionEvaluator permissionEvaluator;

	// Context initialization
	static {
		ApplicationContext context = CurrentApplicationContext.getContext();
		if (context != null) {
			try {
				permissionEvaluator = context.getBean(PermissionEvaluator.class);
			} catch (BeansException e) {
				LOG.warn("Could not find PermissionEvaluator instance in your context");
			}
		} else {
			LOG.warn("You should initialize a bean instance of org.genesys.blocks.util.CurrentApplicationContext in your context");
		}
	}

	/**
	 * Instantiates a new current permissions writer.
	 */
	public CurrentPermissionsWriter() {
		LOG.trace("CurrentPermissionsWriter");
	}

	/**
	 * Instantiates a new current permissions writer.
	 *
	 * @param propDef the prop def
	 * @param annotations the annotations
	 * @param type the type
	 */
	public CurrentPermissionsWriter(BeanPropertyDefinition propDef, Annotations annotations, JavaType type) {
		super(propDef, annotations, type);
		LOG.trace("CurrentPermissionsWriter");
	}

	/*
	 * (non-Javadoc)
	 * @see
	 * com.fasterxml.jackson.databind.ser.VirtualBeanPropertyWriter#value(java.lang.
	 * Object, com.fasterxml.jackson.core.JsonGenerator,
	 * com.fasterxml.jackson.databind.SerializerProvider)
	 */
	@Override
	protected Object value(Object bean, JsonGenerator gen, SerializerProvider prov) throws Exception {
		if (permissionEvaluator != null) {
			Authentication authentication = SecurityContextHolder.getContext().getAuthentication();
			if (authentication != null) {
				CurrentPermissions perms = new CurrentPermissions();
				perms.create = permissionEvaluator.hasPermission(authentication, bean, BasePermission.CREATE);
				perms.read = permissionEvaluator.hasPermission(authentication, bean, BasePermission.READ);
				perms.write = permissionEvaluator.hasPermission(authentication, bean, BasePermission.WRITE);
				perms.delete = permissionEvaluator.hasPermission(authentication, bean, BasePermission.DELETE);
				perms.manage = permissionEvaluator.hasPermission(authentication, bean, BasePermission.ADMINISTRATION);
				return perms;
			}
		}
		return null;
	}

	/*
	 * (non-Javadoc)
	 * @see
	 * com.fasterxml.jackson.databind.ser.VirtualBeanPropertyWriter#withConfig(com.
	 * fasterxml.jackson.databind.cfg.MapperConfig,
	 * com.fasterxml.jackson.databind.introspect.AnnotatedClass,
	 * com.fasterxml.jackson.databind.introspect.BeanPropertyDefinition,
	 * com.fasterxml.jackson.databind.JavaType)
	 */
	@Override
	public VirtualBeanPropertyWriter withConfig(MapperConfig<?> config, AnnotatedClass declaringClass, BeanPropertyDefinition propDef, JavaType type) {
		return new CurrentPermissionsWriter(propDef, declaringClass.getAnnotations(), type);
	}

}

==> application-blocks-security-1.6-sources.jar: org/genesys/blocks/security/serialization/CurrentPermissionsWriter.java <==
/*
 * Copyright 2018 Global Crop Diversity Trust
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.genesys.blocks.security.serialization;

import org.genesys.blocks.security.SecurityContextUtil;
import org.genesys.blocks.security.model.AclAwareModel;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.security.acls.domain.BasePermission;
import org.springframework.security.core.Authentication;
import org.springframework.security.core.context.SecurityContextHolder;

import com.fasterxml.jackson.core.JsonGenerator;
import com.fasterxml.jackson.databind.JavaType;
import com.fasterxml.jackson.databind.SerializerProvider;
import com.fasterxml.jackson.databind.cfg.MapperConfig;
import com.fasterxml.jackson.databind.introspect.AnnotatedClass;
import com.fasterxml.jackson.databind.introspect.BeanPropertyDefinition;
import com.fasterxml.jackson.databind.ser.VirtualBeanPropertyWriter;
import com.fasterxml.jackson.databind.util.Annotations;

/**
 * The CurrentPermissionsWriter is applied to {@link AclAwareModel} and it
 * instructs Jackson to to include {@link Permissions} for current SID for every
 * ACL aware entity.
 * 
 * Serialization is enabled <code>@JsonAppend</code> annotation on
 * <code>AclAwareModel</code>:
 * 
 * <pre>
 *  &#64;JsonAppend(props = { @JsonAppend.Prop(name="_permissions", value = CurrentPermissionsWriter.class, type=CurrentPermissions.class) })
 *  public interface AclAwareModel...
 * </pre>
 * 
 * To be able to access the current permissions of the current SID, this code
 * relies on an instance of
 * <code>org.genesys.blocks.util.CurrentApplicationContext</code> to be
 * registered in the Spring application context.
 * 
 */
public class CurrentPermissionsWriter extends VirtualBeanPropertyWriter {

	/** The Constant LOG. */
	private static final Logger LOG = LoggerFactory.getLogger(CurrentPermissionsWriter.class);

	/** The Constant serialVersionUID. */
	private static final long serialVersionUID = 1L;


	private static final Permissions NO_PERMISSIONS = new Permissions().grantNone();

	/**
	 * Instantiates a new current permissions writer.
	 */
	public CurrentPermissionsWriter() {
		LOG.trace("CurrentPermissionsWriter");
	}

	/**
	 * Instantiates a new current permissions writer.
	 *
	 * @param propDef the prop def
	 * @param annotations the annotations
	 * @param type the type
	 */
	public CurrentPermissionsWriter(BeanPropertyDefinition propDef, Annotations annotations, JavaType type) {
		super(propDef, annotations, type);
		LOG.trace("CurrentPermissionsWriter");
	}

	/*
	 * (non-Javadoc)
	 * @see
	 * com.fasterxml.jackson.databind.ser.VirtualBeanPropertyWriter#value(java.lang.
	 * Object, com.fasterxml.jackson.core.JsonGenerator,
	 * com.fasterxml.jackson.databind.SerializerProvider)
	 */
	@Override
	protected Object value(Object bean, JsonGenerator gen, SerializerProvider prov) throws Exception {
		Authentication authentication = SecurityContextHolder.getContext().getAuthentication();
		if (authentication == null) {
			return NO_PERMISSIONS;
		}
		
		Permissions perms = new Permissions();
		try {
			perms.isPublic = SecurityContextUtil.anyoneHasPermission(bean, "READ");
		} catch (Throwable e) {
			LOG.warn("Could not read public permissions {}", e.getMessage());
			perms.isPublic = false;
		}

		if (SecurityContextUtil.hasRole("ADMINISTRATOR")) {
			perms.grantAll();
		} else {
			try {
				perms.create = SecurityContextUtil.hasPermission(bean, BasePermission.CREATE);
				perms.read = SecurityContextUtil.hasPermission(bean, BasePermission.READ);
				perms.write = SecurityContextUtil.hasPermission(bean, BasePermission.WRITE);
				perms.delete = SecurityContextUtil.hasPermission(bean, BasePermission.DELETE);
				perms.manage = SecurityContextUtil.hasPermission(bean, BasePermission.ADMINISTRATION);
			} catch (Throwable e) {
				LOG.warn("Could not read current permissions {}", e.getMessage());
			}
		}
		return perms;
	}

	/*
	 * (non-Javadoc)
	 * @see
	 * com.fasterxml.jackson.databind.ser.VirtualBeanPropertyWriter#withConfig(com.
	 * fasterxml.jackson.databind.cfg.MapperConfig,
	 * com.fasterxml.jackson.databind.introspect.AnnotatedClass,
	 * com.fasterxml.jackson.databind.introspect.BeanPropertyDefinition,
	 * com.fasterxml.jackson.databind.JavaType)
	 */
	@Override
	public VirtualBeanPropertyWriter withConfig(MapperConfig<?> config, AnnotatedClass declaringClass, BeanPropertyDefinition propDef, JavaType type) {
		return new CurrentPermissionsWriter(propDef, declaringClass.getAnnotations(), type);
	}

}
